#pragma once

#pragma region copied_from_boost_uuid_x86
#include <stddef.h>
#include <cstdint>
#include <crtdefs.h>
#include <stdio.h>

using std::uint8_t;

namespace std  {
	typedef ::size_t size_t;
	typedef ::ptrdiff_t ptrdiff_t;
}

#define BOOST_NOEXCEPT /*noexcept*/
#define BOOST_FORCEINLINE __forceinline
#define BOOST_CONSTEXPR /*constexpr*/

// MSVC does not always have immintrin.h (at least, not up to MSVC 10), so include the appropriate header for each instruction set
#if defined(BOOST_UUID_USE_SSE41)
#include <smmintrin.h>
#elif defined(BOOST_UUID_USE_SSE3)
#include <pmmintrin.h>
#else
#include <emmintrin.h>
#endif

namespace boost {
	namespace uuids {
		namespace detail {

			BOOST_FORCEINLINE __m128i load_unaligned_si128(const uint8_t* p) BOOST_NOEXCEPT
			{
#if defined(BOOST_UUID_USE_SSE3)
				return _mm_lddqu_si128(reinterpret_cast<const __m128i*>(p));
#else
				return _mm_loadu_si128(reinterpret_cast<const __m128i*>(p));
#endif
			}

		} // namespace detail
	}
}


namespace boost {
	namespace uuids {

		struct uuid
		{
		public:
			// or should it be array<uint8_t, 16>
			uint8_t data[16];
		};

		bool operator== (uuid const& lhs, uuid const& rhs) BOOST_NOEXCEPT;
		bool operator< (uuid const& lhs, uuid const& rhs) BOOST_NOEXCEPT;


inline bool operator== (uuid const& lhs, uuid const& rhs) BOOST_NOEXCEPT
{
	register __m128i mm_left = uuids::detail::load_unaligned_si128(lhs.data);
	register __m128i mm_right = uuids::detail::load_unaligned_si128(rhs.data);

	register __m128i mm_cmp = _mm_cmpeq_epi32(mm_left, mm_right);
#if defined(BOOST_UUID_USE_SSE41)
	return _mm_test_all_ones(mm_cmp);
#else
	return _mm_movemask_epi8(mm_cmp) == 0xFFFF;
#endif
}

inline bool operator!=(uuid const& lhs, uuid const& rhs) BOOST_NOEXCEPT
{
	return !(lhs == rhs);
}
}}

using boost::uuids::uuid;
#pragma endregion copied_from_boost_uuid_x86

class headerProperty
{
public:
	virtual ~headerProperty() {}
};

class my_obj:public headerProperty
{
public:
	// This char tmp[8] forces the following uuid to be misaligned.
	char tmp[8];
	// This m_uuid is misaligned (not 16-byte aligned) and causes the != operator to crash.
	uuid m_uuid;
	const uuid &get_marker_id() const { return m_uuid;  }
	uuid get_id() const { return m_uuid;  }
};


